@@ -26,6 +26,7 @@
 #include "libtcmu.h"
 #include "tcmur_device.h"
 #include "tcmur_cmd_handler.h"
+#include "version.h"
 
 #define ALLOWED_BSOFLAGS (O_DIRECT | O_RDWR | O_LARGEFILE)
 
@@ -80,13 +81,14 @@ struct glfs_state {
 
 typedef struct glfs_cbk_cookie {
 	struct tcmu_device *dev;
-	struct tcmulib_cmd *cmd;
+	struct tcmur_cmd *tcmur_cmd;
 	size_t length;
 	enum {
 		TCMU_GLFS_READ  = 1,
 		TCMU_GLFS_WRITE = 2,
 		TCMU_GLFS_FLUSH = 3,
-		TCMU_GLFS_DISCARD = 4
+		TCMU_GLFS_DISCARD = 4,
+		TCMU_GLFS_WRITESAME = 5
 	} op;
 } glfs_cbk_cookie;
 
@@ -99,7 +101,6 @@ struct gluster_cacheconn {
 
 static darray(struct gluster_cacheconn *) glfs_cache = darray_new();
 
-
 const char *const gluster_transport_lookup[] = {
 	[GLUSTER_TRANSPORT_TCP] = "tcp",
 	[GLUSTER_TRANSPORT_UNIX] = "unix",
@@ -107,7 +108,6 @@ const char *const gluster_transport_lookup[] = {
 	[GLUSTER_TRANSPORT__MAX] = NULL,
 };
 
-
 static void gluster_free_host(gluster_hostdef *host)
 {
 	if(!host)
@@ -134,19 +134,19 @@ gluster_compare_hosts(gluster_hostdef *src_server, gluster_hostdef *dst_server)
 		return false;
 
 	switch (src_server->type) {
-		case GLUSTER_TRANSPORT_UNIX:
-			if (!strcmp(src_server->u.uds.socket, dst_server->u.uds.socket))
-				return true;
-			break;
-		case GLUSTER_TRANSPORT_TCP:
-		case GLUSTER_TRANSPORT_RDMA:
-			if (!strcmp(src_server->u.inet.addr, dst_server->u.inet.addr)
-					&&
-				!strcmp(src_server->u.inet.port, dst_server->u.inet.port))
-				return true;
-			break;
-		case GLUSTER_TRANSPORT__MAX:
-			break;
+	case GLUSTER_TRANSPORT_UNIX:
+		if (!strcmp(src_server->u.uds.socket, dst_server->u.uds.socket))
+			return true;
+		break;
+	case GLUSTER_TRANSPORT_TCP:
+	case GLUSTER_TRANSPORT_RDMA:
+		if (!strcmp(src_server->u.inet.addr, dst_server->u.inet.addr)
+				&&
+			!strcmp(src_server->u.inet.port, dst_server->u.inet.port))
+			return true;
+		break;
+	case GLUSTER_TRANSPORT__MAX:
+		break;
 	}
 
 	return false;
@@ -194,6 +194,29 @@ error:
 	return -1;
 }
 
+static bool tcmu_glfs_update_logdir(void)
+{
+	struct gluster_cacheconn **entry;
+	char logfilepath[PATH_MAX];
+	int ret;
+
+	darray_foreach(entry, glfs_cache) {
+		ret = tcmu_make_absolute_logfile(logfilepath, TCMU_GLFS_LOG_FILENAME);
+		if (ret < 0) {
+			tcmu_err("tcmu_make_absolute_logfile failed: %d\n", ret);
+			return false;
+		}
+
+		if (glfs_set_logging((*entry)->fs, logfilepath, TCMU_GLFS_DEBUG_LEVEL)) {
+			tcmu_err("glfs_set_logging() on %s failed[%s]",
+				 (*entry)->volname, strerror(errno));
+			return false;
+		}
+	}
+
+	return true;
+}
+
 static glfs_t* gluster_cache_query(gluster_server *dst, char *cfgstring)
 {
 	struct gluster_cacheconn **entry;
@@ -418,9 +441,8 @@ static glfs_t* tcmu_create_glfs_object(struct tcmu_device *dev,
 		goto fail;
 	}
 
-	if (!init) {
+	if (!init)
 		return fs;
-	}
 
 	ret = glfs_set_volfile_server(fs,
 				gluster_transport_lookup[entry->server->type],
@@ -465,7 +487,7 @@ static char* tcmu_get_path( struct tcmu_device *dev)
 {
 	char *config;
 
-	config = strchr(tcmu_get_dev_cfgstring(dev), '/');
+	config = strchr(tcmu_dev_get_cfgstring(dev), '/');
 	if (!config) {
 		tcmu_dev_err(dev, "no configuration found in cfgstring\n");
 		return NULL;
@@ -482,19 +504,18 @@ static int tcmu_glfs_open(struct tcmu_device *dev, bool reopen)
 	struct stat st;
 	int ret = -EIO;
 	long long dev_size;
-	uint32_t block_size = tcmu_get_dev_block_size(dev);
+	uint32_t block_size = tcmu_dev_get_block_size(dev);
 
 	gfsp = calloc(1, sizeof(*gfsp));
 	if (!gfsp)
 		return -ENOMEM;
 
-	tcmu_set_dev_private(dev, gfsp);
-	tcmu_set_dev_write_cache_enabled(dev, 1);
+	tcmur_dev_set_private(dev, gfsp);
+	tcmu_dev_set_write_cache_enabled(dev, 1);
 
 	config = tcmu_get_path(dev);
-	if (!config) {
+	if (!config)
 		goto fail;
-	}
 
 	gfsp->fs = tcmu_create_glfs_object(dev, config, &gfsp->hosts);
 	if (!gfsp->fs) {
@@ -515,7 +536,7 @@ static int tcmu_glfs_open(struct tcmu_device *dev, bool reopen)
 		goto close;
 	}
 
-	dev_size = tcmu_get_dev_num_lbas(dev) * block_size;
+	dev_size = tcmu_dev_get_num_lbas(dev) * block_size;
 	if (st.st_size != dev_size) {
 		/*
 		 * The glfs allows the backend file size not to align
@@ -559,7 +580,7 @@ fail:
 
 static void tcmu_glfs_close(struct tcmu_device *dev)
 {
-	struct glfs_state *gfsp = tcmu_get_dev_private(dev);
+	struct glfs_state *gfsp = tcmur_dev_get_private(dev);
 
 	glfs_close(gfsp->gfd);
 	gluster_cache_refresh(gfsp->fs, tcmu_get_path(dev));
@@ -567,11 +588,18 @@ static void tcmu_glfs_close(struct tcmu_device *dev)
 	free(gfsp);
 }
 
+#if GFAPI_VERSION760
+static void glfs_async_cbk(glfs_fd_t *fd, ssize_t ret,
+			   struct glfs_stat *prestat,
+			   struct glfs_stat *poststat,
+			   void *data)
+#else
 static void glfs_async_cbk(glfs_fd_t *fd, ssize_t ret, void *data)
+#endif
 {
 	glfs_cbk_cookie *cookie = data;
 	struct tcmu_device *dev = cookie->dev;
-	struct tcmulib_cmd *cmd = cookie->cmd;
+	struct tcmur_cmd *tcmur_cmd = cookie->tcmur_cmd;
 	size_t length = cookie->length;
 
 	if (ret < 0 || ret != length) {
@@ -583,6 +611,7 @@ static void glfs_async_cbk(glfs_fd_t *fd, ssize_t ret, void *data)
 		case TCMU_GLFS_WRITE:
 		case TCMU_GLFS_FLUSH:
 		case TCMU_GLFS_DISCARD:
+		case TCMU_GLFS_WRITESAME:
 			ret = TCMU_STS_WR_ERR;
 			break;
 		}
@@ -590,16 +619,16 @@ static void glfs_async_cbk(glfs_fd_t *fd, ssize_t ret, void *data)
 		ret = TCMU_STS_OK;
 	}
 
-	cmd->done(dev, cmd, ret);
+	tcmur_cmd_complete(dev, tcmur_cmd, ret);
 	free(cookie);
 }
 
 static int tcmu_glfs_read(struct tcmu_device *dev,
-                          struct tcmulib_cmd *cmd,
+                          struct tcmur_cmd *tcmur_cmd,
                           struct iovec *iov, size_t iov_cnt,
                           size_t length, off_t offset)
 {
-	struct glfs_state *state = tcmu_get_dev_private(dev);
+	struct glfs_state *state = tcmur_dev_get_private(dev);
 	glfs_cbk_cookie *cookie;
 
 	cookie = calloc(1, sizeof(*cookie));
@@ -608,7 +637,7 @@ static int tcmu_glfs_read(struct tcmu_device *dev,
 		goto out;
 	}
 	cookie->dev = dev;
-	cookie->cmd = cmd;
+	cookie->tcmur_cmd = tcmur_cmd;
 	cookie->length = length;
 	cookie->op = TCMU_GLFS_READ;
 
@@ -627,11 +656,11 @@ out:
 }
 
 static int tcmu_glfs_write(struct tcmu_device *dev,
-                           struct tcmulib_cmd *cmd,
+                           struct tcmur_cmd *tcmur_cmd,
                            struct iovec *iov, size_t iov_cnt,
                            size_t length, off_t offset)
 {
-	struct glfs_state *state = tcmu_get_dev_private(dev);
+	struct glfs_state *state = tcmur_dev_get_private(dev);
 	glfs_cbk_cookie *cookie;
 
 	cookie = calloc(1, sizeof(*cookie));
@@ -640,7 +669,7 @@ static int tcmu_glfs_write(struct tcmu_device *dev,
 		goto out;
 	}
 	cookie->dev = dev;
-	cookie->cmd = cmd;
+	cookie->tcmur_cmd = tcmur_cmd;
 	cookie->length = length;
 	cookie->op = TCMU_GLFS_WRITE;
 
@@ -661,7 +690,7 @@ out:
 static int tcmu_glfs_reconfig(struct tcmu_device *dev,
                               struct tcmulib_cfg_info *cfg)
 {
-	struct glfs_state *gfsp = tcmu_get_dev_private(dev);
+	struct glfs_state *gfsp = tcmur_dev_get_private(dev);
 	struct stat st;
 	int ret = -EIO;
 
@@ -689,9 +718,9 @@ static int tcmu_glfs_reconfig(struct tcmu_device *dev,
 }
 
 static int tcmu_glfs_flush(struct tcmu_device *dev,
-                           struct tcmulib_cmd *cmd)
+                           struct tcmur_cmd *tcmur_cmd)
 {
-	struct glfs_state *state = tcmu_get_dev_private(dev);
+	struct glfs_state *state = tcmur_dev_get_private(dev);
 	glfs_cbk_cookie *cookie;
 
 	cookie = calloc(1, sizeof(*cookie));
@@ -700,7 +729,7 @@ static int tcmu_glfs_flush(struct tcmu_device *dev,
 		goto out;
 	}
 	cookie->dev = dev;
-	cookie->cmd = cmd;
+	cookie->tcmur_cmd = tcmur_cmd;
 	cookie->length = 0;
 	cookie->op = TCMU_GLFS_FLUSH;
 
@@ -717,10 +746,11 @@ out:
 	return TCMU_STS_NO_RESOURCE;
 }
 
-static int tcmu_glfs_discard(struct tcmu_device *dev, struct tcmulib_cmd *cmd,
+static int tcmu_glfs_discard(struct tcmu_device *dev,
+			     struct tcmur_cmd *tcmur_cmd,
                              uint64_t offset, uint64_t length)
 {
-	struct glfs_state *state = tcmu_get_dev_private(dev);
+	struct glfs_state *state = tcmur_dev_get_private(dev);
 	glfs_cbk_cookie *cookie;
 	ssize_t ret;
 
@@ -730,7 +760,7 @@ static int tcmu_glfs_discard(struct tcmu_device *dev, struct tcmulib_cmd *cmd,
 		goto out;
 	}
 	cookie->dev = dev;
-	cookie->cmd = cmd;
+	cookie->tcmur_cmd = tcmur_cmd;
 	cookie->length = 0;
 	cookie->op = TCMU_GLFS_DISCARD;
 
@@ -748,6 +778,68 @@ out:
 	return TCMU_STS_NO_RESOURCE;
 }
 
+static int tcmu_glfs_writesame(struct tcmu_device *dev,
+			       struct tcmur_cmd *tcmur_cmd,
+			       uint64_t offset, uint64_t length,
+			       struct iovec *iov, size_t iov_cnt)
+{
+	struct glfs_state *state = tcmur_dev_get_private(dev);
+	glfs_cbk_cookie *cookie;
+	ssize_t ret;
+
+	if (!tcmu_iovec_zeroed(iov, iov_cnt)) {
+		tcmu_dev_warn(dev,
+			      "Received none zeroed data, will fall back to writesame emulator instead.\n");
+		return TCMU_STS_NOT_HANDLED;
+	}
+
+	cookie = calloc(1, sizeof(*cookie));
+	if (!cookie) {
+		tcmu_dev_err(dev, "Could not allocate cookie: %m\n");
+		goto out;
+	}
+	cookie->dev = dev;
+	cookie->tcmur_cmd = tcmur_cmd;
+	cookie->length = 0;
+	cookie->op = TCMU_GLFS_WRITESAME;
+
+	ret = glfs_zerofill_async(state->gfd, offset, length, glfs_async_cbk, cookie);
+	if (ret < 0) {
+		tcmu_dev_err(dev, "glfs_zerofill_async(vol=%s, file=%s) failed: %m\n",
+		             state->hosts->volname, state->hosts->path);
+		goto out;
+	}
+
+	return TCMU_STS_OK;
+
+out:
+	free(cookie);
+	return TCMU_STS_NO_RESOURCE;
+}
+
+/*
+ * Return scsi status or TCMU_STS_NOT_HANDLED
+ */
+static int tcmu_glfs_handle_cmd(struct tcmu_device *dev,
+				struct tcmur_cmd *tcmur_cmd)
+{
+	struct tcmulib_cmd *cmd = tcmur_cmd->lib_cmd;
+	uint8_t *cdb = cmd->cdb;
+	int ret;
+
+	switch(cdb[0]) {
+	case WRITE_SAME:
+	case WRITE_SAME_16:
+		ret = tcmur_handle_writesame(dev, tcmur_cmd,
+					     tcmu_glfs_writesame);
+		break;
+	default:
+		ret = TCMU_STS_NOT_HANDLED;
+	}
+
+	return ret;
+}
+
 /*
  * For backstore creation
  *
@@ -780,6 +872,9 @@ struct tcmur_handler glfs_handler = {
 	.reconfig       = tcmu_glfs_reconfig,
 	.flush          = tcmu_glfs_flush,
 	.unmap          = tcmu_glfs_discard,
+	.handle_cmd     = tcmu_glfs_handle_cmd,
+
+	.update_logdir  = tcmu_glfs_update_logdir,
 };
 
 /* Entry point must be named "handler_init". */
@@ -788,14 +883,12 @@ int handler_init(void)
 	int ret;
 
 	ret = pthread_mutex_init(&glfs_lock, NULL);
-	if (ret != 0) {
+	if (ret != 0)
 		return -1;
-	}
 
 	ret = tcmur_register_handler(&glfs_handler);
-	if (ret != 0) {
+	if (ret != 0)
 		pthread_mutex_destroy(&glfs_lock);
-	}
 
 	return ret;
 }
