@@ -90,7 +90,7 @@ enum rbd_aio_type {
 
 struct rbd_aio_cb {
 	struct tcmu_device *dev;
-	struct tcmulib_cmd *tcmulib_cmd;
+	struct tcmur_cmd *tcmur_cmd;
 
 	enum rbd_aio_type type;
 	union {
@@ -113,7 +113,7 @@ struct rbd_aio_cb {
 static void tcmu_rbd_service_status_update(struct tcmu_device *dev,
 					   bool has_lock)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	char *status_buf = NULL;
 	int ret;
 
@@ -136,7 +136,7 @@ static void tcmu_rbd_service_status_update(struct tcmu_device *dev,
 
 static int tcmu_rbd_service_register(struct tcmu_device *dev)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	struct utsname u;
 	char *daemon_buf = NULL;
 	char *metadata_buf = NULL;
@@ -240,7 +240,7 @@ static bool tcmu_rbd_match_device_class(struct tcmu_device *dev,
 					const char *crush_rule,
 					const char *device_class)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	char *mon_cmd_bufs[2] = {NULL, NULL};
 	char *mon_buf = NULL, *mon_status_buf = NULL;
 	size_t mon_buf_len = 0, mon_status_buf_len = 0;
@@ -280,7 +280,7 @@ static bool tcmu_rbd_match_device_class(struct tcmu_device *dev,
 
 static void tcmu_rbd_detect_device_class(struct tcmu_device *dev)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	char *mon_cmd_bufs[2] = {NULL, NULL};
 	char *mon_buf = NULL, *mon_status_buf = NULL;
 	size_t mon_buf_len = 0, mon_status_buf_len = 0;
@@ -337,7 +337,7 @@ static void tcmu_rbd_detect_device_class(struct tcmu_device *dev)
 	    tcmu_rbd_match_device_class(dev, crush_rule, "nvme")) {
 		tcmu_dev_dbg(dev, "Pool %s associated to solid state device class.\n",
 			     state->pool_name);
-		tcmu_set_dev_solid_state_media(dev, true);
+		tcmu_dev_set_solid_state_media(dev, true);
 	}
 
 	free(crush_rule);
@@ -345,7 +345,7 @@ static void tcmu_rbd_detect_device_class(struct tcmu_device *dev)
 
 static void tcmu_rbd_image_close(struct tcmu_device *dev)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 
 	rbd_close(state->image);
 	rados_ioctx_destroy(state->io_ctx);
@@ -358,7 +358,7 @@ static void tcmu_rbd_image_close(struct tcmu_device *dev)
 
 static int timer_check_and_set_def(struct tcmu_device *dev)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	char buf[128];
 	int grace, interval, ret, len;
 	float timeout;
@@ -429,7 +429,7 @@ set:
 
 static int tcmu_rbd_image_open(struct tcmu_device *dev)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	int ret;
 
 	ret = rados_create(&state->cluster, state->id);
@@ -507,15 +507,17 @@ rados_shutdown:
  */
 static int tcmu_rbd_has_lock(struct tcmu_device *dev)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	int ret, is_owner;
 
 	ret = rbd_is_exclusive_lock_owner(state->image, &is_owner);
-	if (ret == -ESHUTDOWN || ret == -ETIMEDOUT) {
-		return ret;
-	} else if (ret < 0) {
+	if (ret < 0) {
+		tcmu_dev_err(dev, "Could not check lock ownership. Error: %s.\n",
+			     strerror(-ret));
+		if (ret == -ESHUTDOWN || ret == -ETIMEDOUT)
+			return ret;
+
 		/* let initiator figure things out */
-		tcmu_dev_err(dev, "Could not check lock ownership. (Err %d).\n", ret);
 		return -EIO;
 	} else if (is_owner) {
 		tcmu_dev_dbg(dev, "Is owner\n");
@@ -550,7 +552,7 @@ static int tcmu_rbd_get_lock_state(struct tcmu_device *dev)
  */
 static int tcmu_rbd_lock_break(struct tcmu_device *dev)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	rbd_lock_mode_t lock_mode;
 	char *owners[1];
 	size_t num_owners = 1;
@@ -604,7 +606,7 @@ static int tcmu_rbd_to_sts(int rc)
 
 static int tcmu_rbd_get_lock_tag(struct tcmu_device *dev, uint16_t *tag)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	char *metadata_owner, *owners[1];
 	size_t num_owners = 1;
 	rbd_lock_mode_t lock_mode;
@@ -671,7 +673,7 @@ done:
 
 static int tcmu_rbd_set_lock_tag(struct tcmu_device *dev, uint16_t tcmu_tag)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	rbd_lock_mode_t lock_mode;
 	char *owners[1];
 	size_t num_owners = 1;
@@ -715,11 +717,14 @@ free_owners:
 
 static int tcmu_rbd_unlock(struct tcmu_device *dev)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	int ret;
 
-	if (tcmu_rbd_has_lock(dev) != 1)
+	ret = tcmu_rbd_has_lock(dev);
+	if (ret == 0)
 		return TCMU_STS_OK;
+	else if (ret < 0)
+		return tcmu_rbd_to_sts(ret);
 
 	ret = rbd_lock_release(state->image);
 	if (!ret)
@@ -731,7 +736,7 @@ static int tcmu_rbd_unlock(struct tcmu_device *dev)
 
 static int tcmu_rbd_lock(struct tcmu_device *dev, uint16_t tag)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	int ret;
 
 	ret = tcmu_rbd_has_lock(dev);
@@ -765,7 +770,7 @@ done:
 
 static void tcmu_rbd_check_excl_lock_enabled(struct tcmu_device *dev)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	uint64_t features = 0;
 	int ret;
 
@@ -806,7 +811,7 @@ static void tcmu_rbd_state_free(struct tcmu_rbd_state *state)
 
 static int tcmu_rbd_check_image_size(struct tcmu_device *dev, uint64_t new_size)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	uint64_t rbd_size;
 	int ret;
 
@@ -838,9 +843,9 @@ static int tcmu_rbd_open(struct tcmu_device *dev, bool reopen)
 	state = calloc(1, sizeof(*state));
 	if (!state)
 		return -ENOMEM;
-	tcmu_set_dev_private(dev, state);
+	tcmur_dev_set_private(dev, state);
 
-	dev_cfg_dup = strdup(tcmu_get_dev_cfgstring(dev));
+	dev_cfg_dup = strdup(tcmu_dev_get_cfgstring(dev));
 	config = dev_cfg_dup;
 	if (!config) {
 		ret = -ENOMEM;
@@ -848,8 +853,8 @@ static int tcmu_rbd_open(struct tcmu_device *dev, bool reopen)
 	}
 
 	tcmu_dev_dbg(dev, "tcmu_rbd_open config %s block size %u num lbas %" PRIu64 ".\n",
-		     config, tcmu_get_dev_block_size(dev),
-		     tcmu_get_dev_num_lbas(dev));
+		     config, tcmu_dev_get_block_size(dev),
+		     tcmu_dev_get_num_lbas(dev));
 
 	config = strchr(config, '/');
 	if (!config) {
@@ -922,8 +927,8 @@ static int tcmu_rbd_open(struct tcmu_device *dev, bool reopen)
 
 	tcmu_rbd_check_excl_lock_enabled(dev);
 
-	ret = tcmu_rbd_check_image_size(dev, tcmu_get_dev_block_size(dev) *
-					tcmu_get_dev_num_lbas(dev));
+	ret = tcmu_rbd_check_image_size(dev, tcmu_dev_get_block_size(dev) *
+					tcmu_dev_get_num_lbas(dev));
 	if (ret) {
 		goto stop_image;
 	}
@@ -940,12 +945,12 @@ static int tcmu_rbd_open(struct tcmu_device *dev, bool reopen)
 	 * overflowing the OSD/ceph side queues with discards/RWs limit it to
 	 * up to 4.
 	 */
-	max_blocks = (image_info.obj_size * 4) / tcmu_get_dev_block_size(dev);
-	tcmu_set_dev_opt_xcopy_rw_len(dev, max_blocks);
-	tcmu_set_dev_max_unmap_len(dev, max_blocks);
-	tcmu_set_dev_opt_unmap_gran(dev, image_info.obj_size /
-				    tcmu_get_dev_block_size(dev), false);
-	tcmu_set_dev_write_cache_enabled(dev, 0);
+	max_blocks = (image_info.obj_size * 4) / tcmu_dev_get_block_size(dev);
+	tcmu_dev_set_opt_xcopy_rw_len(dev, max_blocks);
+	tcmu_dev_set_max_unmap_len(dev, max_blocks);
+	tcmu_dev_set_opt_unmap_gran(dev, image_info.obj_size /
+				    tcmu_dev_get_block_size(dev), false);
+	tcmu_dev_set_write_cache_enabled(dev, 0);
 
 	free(dev_cfg_dup);
 	return 0;
@@ -961,15 +966,14 @@ free_state:
 
 static void tcmu_rbd_close(struct tcmu_device *dev)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 
 	tcmu_rbd_image_close(dev);
 	tcmu_rbd_state_free(state);
 }
 
-static int tcmu_rbd_handle_blacklisted_cmd(struct tcmu_device *dev,
-					   struct tcmulib_cmd *cmd)
-{
+static int tcmu_rbd_handle_blacklisted_cmd(struct tcmu_device *dev)
+		{
 	tcmu_notify_lock_lost(dev);
 	/*
 	 * This will happen during failback normally, because
@@ -986,9 +990,8 @@ static int tcmu_rbd_handle_blacklisted_cmd(struct tcmu_device *dev,
  * we will end up failing the transport connection when we just needed
  * to try a different OSD.
  */
-static int tcmu_rbd_handle_timedout_cmd(struct tcmu_device *dev,
-					struct tcmulib_cmd *cmd)
-{
+static int tcmu_rbd_handle_timedout_cmd(struct tcmu_device *dev)
+		{
 	tcmu_dev_err(dev, "Timing out cmd.\n");
 	tcmu_notify_conn_lost(dev);
 
@@ -1009,7 +1012,7 @@ static int tcmu_rbd_handle_timedout_cmd(struct tcmu_device *dev,
 
 static rbd_image_t tcmu_dev_to_image(struct tcmu_device *dev)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	return state->image;
 }
 
@@ -1025,7 +1028,7 @@ static int tcmu_rbd_aio_read(struct tcmu_device *dev, struct rbd_aio_cb *aio_cb,
 			     rbd_completion_t completion, struct iovec *iov,
 			     size_t iov_cnt, size_t length, off_t offset)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	int ret;
 
 	aio_cb->bounce_buffer = malloc(length);
@@ -1045,7 +1048,7 @@ static int tcmu_rbd_aio_write(struct tcmu_device *dev, struct rbd_aio_cb *aio_cb
 			      rbd_completion_t completion, struct iovec *iov,
 			      size_t iov_cnt, size_t length, off_t offset)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	int ret;
 
 	aio_cb->bounce_buffer = malloc(length);
@@ -1075,7 +1078,7 @@ static void rbd_finish_aio_generic(rbd_completion_t completion,
 				   struct rbd_aio_cb *aio_cb)
 {
 	struct tcmu_device *dev = aio_cb->dev;
-	struct tcmulib_cmd *tcmulib_cmd = aio_cb->tcmulib_cmd;
+	struct tcmur_cmd *tcmur_cmd = aio_cb->tcmur_cmd;
 	struct iovec *iov = aio_cb->iov;
 	size_t iov_cnt = aio_cb->iov_cnt;
 	uint32_t cmp_offset;
@@ -1086,15 +1089,15 @@ static void rbd_finish_aio_generic(rbd_completion_t completion,
 	rbd_aio_release(completion);
 
 	if (ret == -ETIMEDOUT) {
-		tcmu_r = tcmu_rbd_handle_timedout_cmd(dev, tcmulib_cmd);
+		tcmu_r = tcmu_rbd_handle_timedout_cmd(dev);
 	} else if (ret == -ESHUTDOWN || ret == -EROFS) {
-		tcmu_r = tcmu_rbd_handle_blacklisted_cmd(dev, tcmulib_cmd);
+		tcmu_r = tcmu_rbd_handle_blacklisted_cmd(dev);
 	} else if (ret == -EILSEQ && aio_cb->type == RBD_AIO_TYPE_CAW) {
 		cmp_offset = aio_cb->caw.miscompare_offset - aio_cb->caw.offset;
 		tcmu_dev_dbg(dev, "CAW miscompare at offset %u.\n", cmp_offset);
 
 		tcmu_r = TCMU_STS_MISCOMPARE;
-		tcmu_set_sense_info(tcmulib_cmd->sense_buf, cmp_offset);
+		tcmu_sense_set_info(tcmur_cmd->lib_cmd->sense_buf, cmp_offset);
 	} else if (ret == -EINVAL) {
 		tcmu_dev_err(dev, "Invalid IO request.\n");
 		tcmu_r = TCMU_STS_INVALID_CDB;
@@ -1115,14 +1118,14 @@ static void rbd_finish_aio_generic(rbd_completion_t completion,
 		}
 	}
 
-	tcmulib_cmd->done(dev, tcmulib_cmd, tcmu_r);
+	tcmur_cmd_complete(dev, tcmur_cmd, tcmu_r);
 
 	if (aio_cb->bounce_buffer)
 		free(aio_cb->bounce_buffer);
 	free(aio_cb);
 }
 
-static int tcmu_rbd_read(struct tcmu_device *dev, struct tcmulib_cmd *cmd,
+static int tcmu_rbd_read(struct tcmu_device *dev, struct tcmur_cmd *tcmur_cmd,
 			     struct iovec *iov, size_t iov_cnt, size_t length,
 			     off_t offset)
 {
@@ -1139,7 +1142,7 @@ static int tcmu_rbd_read(struct tcmu_device *dev, struct tcmulib_cmd *cmd,
 	aio_cb->dev = dev;
 	aio_cb->type = RBD_AIO_TYPE_READ;
 	aio_cb->read.length = length;
-	aio_cb->tcmulib_cmd = cmd;
+	aio_cb->tcmur_cmd = tcmur_cmd;
 	aio_cb->iov = iov;
 	aio_cb->iov_cnt = iov_cnt;
 
@@ -1164,7 +1167,7 @@ out:
 	return TCMU_STS_NO_RESOURCE;
 }
 
-static int tcmu_rbd_write(struct tcmu_device *dev, struct tcmulib_cmd *cmd,
+static int tcmu_rbd_write(struct tcmu_device *dev, struct tcmur_cmd *tcmur_cmd,
 			  struct iovec *iov, size_t iov_cnt, size_t length,
 			  off_t offset)
 {
@@ -1180,7 +1183,7 @@ static int tcmu_rbd_write(struct tcmu_device *dev, struct tcmulib_cmd *cmd,
 
 	aio_cb->dev = dev;
 	aio_cb->type = RBD_AIO_TYPE_WRITE;
-	aio_cb->tcmulib_cmd = cmd;
+	aio_cb->tcmur_cmd = tcmur_cmd;
 
 	ret = rbd_aio_create_completion
 		(aio_cb, (rbd_callback_t) rbd_finish_aio_generic, &completion);
@@ -1205,10 +1208,10 @@ out:
 }
 
 #ifdef RBD_DISCARD_SUPPORT
-static int tcmu_rbd_unmap(struct tcmu_device *dev, struct tcmulib_cmd *cmd,
+static int tcmu_rbd_unmap(struct tcmu_device *dev, struct tcmur_cmd *tcmur_cmd,
 			  uint64_t off, uint64_t len)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	struct rbd_aio_cb *aio_cb;
 	rbd_completion_t completion;
 	ssize_t ret;
@@ -1220,7 +1223,7 @@ static int tcmu_rbd_unmap(struct tcmu_device *dev, struct tcmulib_cmd *cmd,
 	}
 
 	aio_cb->dev = dev;
-	aio_cb->tcmulib_cmd = cmd;
+	aio_cb->tcmur_cmd = tcmur_cmd;
 	aio_cb->type = RBD_AIO_TYPE_WRITE;
 	aio_cb->bounce_buffer = NULL;
 
@@ -1246,9 +1249,9 @@ out:
 
 #ifdef LIBRBD_SUPPORTS_AIO_FLUSH
 
-static int tcmu_rbd_flush(struct tcmu_device *dev, struct tcmulib_cmd *cmd)
+static int tcmu_rbd_flush(struct tcmu_device *dev, struct tcmur_cmd *tcmur_cmd)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	struct rbd_aio_cb *aio_cb;
 	rbd_completion_t completion;
 	ssize_t ret;
@@ -1260,7 +1263,7 @@ static int tcmu_rbd_flush(struct tcmu_device *dev, struct tcmulib_cmd *cmd)
 	}
 
 	aio_cb->dev = dev;
-	aio_cb->tcmulib_cmd = cmd;
+	aio_cb->tcmur_cmd = tcmur_cmd;
 	aio_cb->type = RBD_AIO_TYPE_WRITE;
 	aio_cb->bounce_buffer = NULL;
 
@@ -1289,11 +1292,11 @@ out:
 
 #ifdef RBD_WRITE_SAME_SUPPORT
 static int tcmu_rbd_aio_writesame(struct tcmu_device *dev,
-				  struct tcmulib_cmd *cmd,
+				  struct tcmur_cmd *tcmur_cmd,
 				  uint64_t off, uint64_t len,
 				  struct iovec *iov, size_t iov_cnt)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	struct rbd_aio_cb *aio_cb;
 	rbd_completion_t completion;
 	size_t length = tcmu_iovec_length(iov, iov_cnt);
@@ -1306,7 +1309,7 @@ static int tcmu_rbd_aio_writesame(struct tcmu_device *dev,
 	}
 
 	aio_cb->dev = dev;
-	aio_cb->tcmulib_cmd = cmd;
+	aio_cb->tcmur_cmd = tcmur_cmd;
 	aio_cb->type = RBD_AIO_TYPE_WRITE;
 
 	aio_cb->bounce_buffer = malloc(length);
@@ -1343,11 +1346,11 @@ out:
 #endif /* RBD_WRITE_SAME_SUPPORT */
 
 #ifdef RBD_COMPARE_AND_WRITE_SUPPORT
-static int tcmu_rbd_aio_caw(struct tcmu_device *dev, struct tcmulib_cmd *cmd,
+static int tcmu_rbd_aio_caw(struct tcmu_device *dev, struct tcmur_cmd *tcmur_cmd,
 			    uint64_t off, uint64_t len, struct iovec *iov,
 			    size_t iov_cnt)
 {
-	struct tcmu_rbd_state *state = tcmu_get_dev_private(dev);
+	struct tcmu_rbd_state *state = tcmur_dev_get_private(dev);
 	struct rbd_aio_cb *aio_cb;
 	rbd_completion_t completion;
 	uint64_t buffer_length = 2 * len;
@@ -1360,7 +1363,7 @@ static int tcmu_rbd_aio_caw(struct tcmu_device *dev, struct tcmulib_cmd *cmd,
 	}
 
 	aio_cb->dev = dev;
-	aio_cb->tcmulib_cmd = cmd;
+	aio_cb->tcmur_cmd = tcmur_cmd;
 	aio_cb->type = RBD_AIO_TYPE_CAW;
 	aio_cb->caw.offset = off;
 
@@ -1405,8 +1408,10 @@ out:
 /*
  * Return scsi status or TCMU_STS_NOT_HANDLED
  */
-static int tcmu_rbd_handle_cmd(struct tcmu_device *dev, struct tcmulib_cmd *cmd)
+static int tcmu_rbd_handle_cmd(struct tcmu_device *dev,
+			       struct tcmur_cmd *tcmur_cmd)
 {
+	struct tcmulib_cmd *cmd = tcmur_cmd->lib_cmd;
 	uint8_t *cdb = cmd->cdb;
 	int ret;
 
@@ -1414,12 +1419,13 @@ static int tcmu_rbd_handle_cmd(struct tcmu_device *dev, struct tcmulib_cmd *cmd)
 #ifdef RBD_WRITE_SAME_SUPPORT
 	case WRITE_SAME:
 	case WRITE_SAME_16:
-		ret = tcmur_handle_writesame(dev, cmd, tcmu_rbd_aio_writesame);
+		ret = tcmur_handle_writesame(dev, tcmur_cmd,
+					     tcmu_rbd_aio_writesame);
 		break;
 #endif
 #ifdef RBD_COMPARE_AND_WRITE_SUPPORT
 	case COMPARE_AND_WRITE:
-		ret = tcmur_handle_caw(dev, cmd, tcmu_rbd_aio_caw);
+		ret = tcmur_handle_caw(dev, tcmur_cmd, tcmu_rbd_aio_caw);
 		break;
 #endif
 	default:
